@@ -3,12 +3,12 @@
  * parse_type.c
  *		handle type operations for parser
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/parser/parse_type.c,v 1.50 2002/09/04 20:31:24 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/parser/parse_type.c,v 1.63 2003/10/20 17:25:42 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -57,8 +57,10 @@ LookupTypeName(const TypeName *typename)
 		switch (length(typename->names))
 		{
 			case 1:
-				elog(ERROR, "Improper %%TYPE reference (too few dotted names): %s",
-					 NameListToString(typename->names));
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("improper %%TYPE reference (too few dotted names): %s",
+								NameListToString(typename->names))));
 				break;
 			case 2:
 				rel->relname = strVal(lfirst(typename->names));
@@ -67,17 +69,19 @@ LookupTypeName(const TypeName *typename)
 			case 3:
 				rel->schemaname = strVal(lfirst(typename->names));
 				rel->relname = strVal(lsecond(typename->names));
-				field = strVal(lfirst(lnext(lnext(typename->names))));
+				field = strVal(lthird(typename->names));
 				break;
 			case 4:
 				rel->catalogname = strVal(lfirst(typename->names));
 				rel->schemaname = strVal(lsecond(typename->names));
-				rel->relname = strVal(lfirst(lnext(lnext(typename->names))));
-				field = strVal(lfirst(lnext(lnext(lnext(typename->names)))));
+				rel->relname = strVal(lthird(typename->names));
+				field = strVal(lfourth(typename->names));
 				break;
 			default:
-				elog(ERROR, "Improper %%TYPE reference (too many dotted names): %s",
-					 NameListToString(typename->names));
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("improper %%TYPE reference (too many dotted names): %s",
+								NameListToString(typename->names))));
 				break;
 		}
 
@@ -85,16 +89,20 @@ LookupTypeName(const TypeName *typename)
 		relid = RangeVarGetRelid(rel, false);
 		attnum = get_attnum(relid, field);
 		if (attnum == InvalidAttrNumber)
-			elog(ERROR, "Relation \"%s\" has no column \"%s\"",
-				 rel->relname, field);
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_COLUMN),
+			 errmsg("column \"%s\" of relation \"%s\" does not exist",
+					field, rel->relname)));
 		restype = get_atttype(relid, attnum);
 
 		/* this construct should never have an array indicator */
 		Assert(typename->arrayBounds == NIL);
 
-		/* emit nuisance warning */
-		elog(NOTICE, "%s converted to %s",
-			 TypeNameToString(typename), format_type_be(restype));
+		/* emit nuisance notice */
+		ereport(NOTICE,
+				(errmsg("type reference %s converted to %s",
+						TypeNameToString(typename),
+						format_type_be(restype))));
 	}
 	else
 	{
@@ -153,13 +161,13 @@ TypeNameToString(const TypeName *typename)
 		{
 			if (l != typename->names)
 				appendStringInfoChar(&string, '.');
-			appendStringInfo(&string, "%s", strVal(lfirst(l)));
+			appendStringInfoString(&string, strVal(lfirst(l)));
 		}
 	}
 	else
 	{
 		/* Look up internally-specified type */
-		appendStringInfo(&string, "%s", format_type_be(typename->typeid));
+		appendStringInfoString(&string, format_type_be(typename->typeid));
 	}
 
 	/*
@@ -167,10 +175,10 @@ TypeNameToString(const TypeName *typename)
 	 * LookupTypeName
 	 */
 	if (typename->pct_type)
-		appendStringInfo(&string, "%%TYPE");
+		appendStringInfoString(&string, "%TYPE");
 
 	if (typename->arrayBounds != NIL)
-		appendStringInfo(&string, "[]");
+		appendStringInfoString(&string, "[]");
 
 	return string.data;
 }
@@ -188,11 +196,15 @@ typenameTypeId(const TypeName *typename)
 
 	typoid = LookupTypeName(typename);
 	if (!OidIsValid(typoid))
-		elog(ERROR, "Type \"%s\" does not exist",
-			 TypeNameToString(typename));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("type \"%s\" does not exist",
+						TypeNameToString(typename))));
 	if (!get_typisdefined(typoid))
-		elog(ERROR, "Type \"%s\" is only a shell",
-			 TypeNameToString(typename));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("type \"%s\" is only a shell",
+						TypeNameToString(typename))));
 	return typoid;
 }
 
@@ -210,17 +222,20 @@ typenameType(const TypeName *typename)
 
 	typoid = LookupTypeName(typename);
 	if (!OidIsValid(typoid))
-		elog(ERROR, "Type \"%s\" does not exist",
-			 TypeNameToString(typename));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("type \"%s\" does not exist",
+						TypeNameToString(typename))));
 	tup = SearchSysCache(TYPEOID,
 						 ObjectIdGetDatum(typoid),
 						 0, 0, 0);
-	if (!HeapTupleIsValid(tup))
-		elog(ERROR, "Type \"%s\" does not exist",
-			 TypeNameToString(typename));
+	if (!HeapTupleIsValid(tup)) /* should not happen */
+		elog(ERROR, "cache lookup failed for type %u", typoid);
 	if (!((Form_pg_type) GETSTRUCT(tup))->typisdefined)
-		elog(ERROR, "Type \"%s\" is only a shell",
-			 TypeNameToString(typename));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("type \"%s\" is only a shell",
+						TypeNameToString(typename))));
 	return (Type) tup;
 }
 
@@ -244,7 +259,7 @@ typeidType(Oid id)
 						 ObjectIdGetDatum(id),
 						 0, 0, 0);
 	if (!HeapTupleIsValid(tup))
-		elog(ERROR, "Unable to locate type oid %u in catalog", id);
+		elog(ERROR, "cache lookup failed for type %u", id);
 	return (Type) tup;
 }
 
@@ -252,7 +267,7 @@ typeidType(Oid id)
 Oid
 typeTypeId(Type tp)
 {
-	if (tp == NULL)
+	if (tp == NULL)				/* probably useless */
 		elog(ERROR, "typeTypeId() called with NULL type struct");
 	return HeapTupleGetOid(tp);
 }
@@ -386,7 +401,7 @@ typeidOutfunc(Oid type_id)
 							   ObjectIdGetDatum(type_id),
 							   0, 0, 0);
 	if (!HeapTupleIsValid(typeTuple))
-		elog(ERROR, "typeidOutfunc: Invalid type - oid = %u", type_id);
+		elog(ERROR, "cache lookup failed for type %u", type_id);
 
 	type = (Form_pg_type) GETSTRUCT(typeTuple);
 	outfunc = type->typoutput;
@@ -407,7 +422,7 @@ typeidTypeRelid(Oid type_id)
 							   ObjectIdGetDatum(type_id),
 							   0, 0, 0);
 	if (!HeapTupleIsValid(typeTuple))
-		elog(ERROR, "typeidTypeRelid: Invalid type - oid = %u", type_id);
+		elog(ERROR, "cache lookup failed for type %u", type_id);
 
 	type = (Form_pg_type) GETSTRUCT(typeTuple);
 	result = type->typrelid;
@@ -415,14 +430,21 @@ typeidTypeRelid(Oid type_id)
 	return result;
 }
 
+/*
+ * error context callback for parse failure during parseTypeString()
+ */
+static void
+pts_error_callback(void *arg)
+{
+	const char *str = (const char *) arg;
+
+	errcontext("invalid type name \"%s\"", str);
+}
+
 /*
  * Given a string that is supposed to be a SQL-compatible type declaration,
  * such as "int4" or "integer" or "character varying(32)", parse
  * the string and convert it to a type OID and type modifier.
- *
- * This routine is not currently used by the main backend, but it is
- * exported for use by add-on modules such as plpgsql, in hopes of
- * centralizing parsing knowledge about SQL type declarations.
  */
 void
 parseTypeString(const char *str, Oid *type_id, int32 *typmod)
@@ -433,18 +455,33 @@ parseTypeString(const char *str, Oid *type_id, int32 *typmod)
 	ResTarget  *restarget;
 	TypeCast   *typecast;
 	TypeName   *typename;
+	ErrorContextCallback ptserrcontext;
+
+	/* make sure we give useful error for empty input */
+	if (strspn(str, " \t\n\r\f") == strlen(str))
+		goto fail;
 
 	initStringInfo(&buf);
-	appendStringInfo(&buf, "SELECT (NULL::%s)", str);
+	appendStringInfo(&buf, "SELECT NULL::%s", str);
 
-	raw_parsetree_list = parser(&buf, NULL, 0);
+	/*
+	 * Setup error traceback support in case of ereport() during parse
+	 */
+	ptserrcontext.callback = pts_error_callback;
+	ptserrcontext.arg = (void *) str;
+	ptserrcontext.previous = error_context_stack;
+	error_context_stack = &ptserrcontext;
+
+	raw_parsetree_list = raw_parser(buf.data);
+
+	error_context_stack = ptserrcontext.previous;
 
 	/*
 	 * Make sure we got back exactly what we expected and no more;
 	 * paranoia is justified since the string might contain anything.
 	 */
 	if (length(raw_parsetree_list) != 1)
-		elog(ERROR, "Invalid type name '%s'", str);
+		goto fail;
 	stmt = (SelectStmt *) lfirst(raw_parsetree_list);
 	if (stmt == NULL ||
 		!IsA(stmt, SelectStmt) ||
@@ -455,33 +492,39 @@ parseTypeString(const char *str, Oid *type_id, int32 *typmod)
 		stmt->groupClause != NIL ||
 		stmt->havingClause != NULL ||
 		stmt->sortClause != NIL ||
-		stmt->portalname != NULL ||
 		stmt->limitOffset != NULL ||
 		stmt->limitCount != NULL ||
 		stmt->forUpdate != NIL ||
 		stmt->op != SETOP_NONE)
-		elog(ERROR, "Invalid type name '%s'", str);
+		goto fail;
 	if (length(stmt->targetList) != 1)
-		elog(ERROR, "Invalid type name '%s'", str);
+		goto fail;
 	restarget = (ResTarget *) lfirst(stmt->targetList);
 	if (restarget == NULL ||
 		!IsA(restarget, ResTarget) ||
 		restarget->name != NULL ||
 		restarget->indirection != NIL)
-		elog(ERROR, "Invalid type name '%s'", str);
+		goto fail;
 	typecast = (TypeCast *) restarget->val;
 	if (typecast == NULL ||
 		!IsA(typecast, TypeCast) ||
 		typecast->arg == NULL ||
 		!IsA(typecast->arg, A_Const))
-		elog(ERROR, "Invalid type name '%s'", str);
+		goto fail;
 	typename = typecast->typename;
 	if (typename == NULL ||
 		!IsA(typename, TypeName))
-		elog(ERROR, "Invalid type name '%s'", str);
+		goto fail;
 
 	*type_id = typenameTypeId(typename);
 	*typmod = typename->typmod;
 
 	pfree(buf.data);
+
+	return;
+
+fail:
+	ereport(ERROR,
+			(errcode(ERRCODE_SYNTAX_ERROR),
+			 errmsg("invalid type name \"%s\"", str)));
 }
