@@ -15,7 +15,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_archiver.c,v 1.79.2.4 2005/04/30 08:36:18 neilc Exp $
+ *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_archiver.c,v 1.79.2.6 2005/08/25 00:11:36 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -944,7 +944,7 @@ SortTocFromFile(Archive *AHX, RestoreOptions *ropt)
 	if (!fh)
 		die_horribly(AH, modulename, "could not open TOC file\n");
 
-	while (fgets(buf, 1024, fh) != NULL)
+	while (fgets(buf, sizeof(buf), fh) != NULL)
 	{
 		/* Find a comment */
 		cmnt = strchr(buf, ';');
@@ -972,10 +972,13 @@ SortTocFromFile(Archive *AHX, RestoreOptions *ropt)
 		if (!te)
 			die_horribly(AH, modulename, "could not find entry for ID %d\n", id);
 
-		ropt->idWanted[id - 1] = 1;
+		if (!ropt->idWanted[id - 1])
+		{
+			ropt->idWanted[id - 1] = 1;
 
-		_moveAfter(AH, tePrev, te);
-		tePrev = te;
+			_moveAfter(AH, tePrev, te);
+			tePrev = te;
+		}
 	}
 
 	if (fclose(fh) != 0)
@@ -2253,7 +2256,7 @@ _printTocEntry(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt, bool isDat
 	 */
 	if (AH->ropt && AH->ropt->noOwner && strcmp(te->desc, "SCHEMA") == 0)
 	{
-		ahprintf(AH, "CREATE SCHEMA %s;\n\n\n", te->tag);
+		ahprintf(AH, "CREATE SCHEMA %s;\n\n\n", fmtId(te->tag));
 	}
 	else
 	{
